#include <stdio.h>
#include "libcomm.h"

#ifndef nothread
  #include <pthread.h>
#endif


#ifndef nothread
/**
   This is a part of socket_accept() and must not called from the
   user. This function is the thread which is started from
   socket_accept() and runs in background.
*/
void thread2(struct LIBCOMMPTHREADP *libcommpthreadp) {
  char *buf;
  unsigned int type;
  unsigned int size;
  /* connector's address information */
  struct sockaddr_in their_addr;
  int sin_size;
  int fd;

  while (1) {
    // wait and accept a incomming connection
    sin_size = sizeof(struct sockaddr_in);
    if ((fd = accept(libcommpthreadp -> sockport,
		     (struct sockaddr *) &their_addr,
		     &sin_size)) != -1) {
      char *pip = inet_ntoa(their_addr.sin_addr);

      libcommpthreadp -> socket_accept_do(fd,
					  libcommpthreadp -> id,
					  pip, their_addr);
    }
  }
  //  pthread_exit(NULL);
}


/**
   Start a new thread, wait for connections and start
   socket_accept_do() when someone connects.

   @param sockport (int) descriptor of a tcp socket/port from
   socket_bind()

   @param id (int) arbitrary id of background process / thread. (May
   be it is a good idea to use the portnumber.)

   @param aocket_accept_do(int fd, int id, char *pip, struct
   sockaddr_in their_addr) (function) this function is called if
   somebody connects.  fd is the descriptor of the new socket to the
   connected tcp-tream. id is the same as in socket_accept(). pip
   contains the ip-address of the connected client. The structure
   their_addr contails all known information about the connected
   client.

   @return If all right zero otherwise non zero.
*/
int socket_accept(int sockport, int id,
	   void (*socket_accept_do)(int fd, int id, char *pip,
				    struct sockaddr_in their_addr)) {

  // allocate memory for thread configuration
  struct LIBCOMMPTHREADP *libcommpthreadp; 
  libcommpthreadp = (struct LIBCOMMPTHREADP *)
    malloc(sizeof(struct LIBCOMMPTHREADP));
  if (libcommpthreadp == NULL) {
    perror("malloc()");
    return -1;
  }

  // store all necessary data in it
  libcommpthreadp -> sockport = sockport;
  libcommpthreadp -> id = id;
  libcommpthreadp -> socket_accept_do = socket_accept_do;

  // starting thread
  pthread_attr_init(&(libcommpthreadp -> thrd_2_attr));
  return pthread_create(&(libcommpthreadp -> thrd_2),
			&(libcommpthreadp -> thrd_2_attr),
			(void *) thread2, libcommpthreadp);

}

#endif
